@@ -24,6 +24,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #endif
 
 #include "jumpurl.h"
+#include "logsmodel.h"
+#include "logswindow.h"
 #include "params.h"
 #include "portgrouplist.h"
 #include "portstatswindow.h"
@@ -36,6 +38,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "fileformat.pb.h"
 
+#include <QDate>
 #include <QDesktopServices>
 #include <QDockWidget>
 #include <QFileDialog>
@@ -56,6 +59,7 @@ extern const char* version;
 extern const char* revision;
 
 PortGroupList    *pgl;
+LogsModel        *appLogs;
 
 MainWindow::MainWindow(QWidget *parent) 
     : QMainWindow (parent)
@@ -89,24 +93,39 @@ MainWindow::MainWindow(QWidget *parent)
         localServer_ = NULL;
 
     pgl = new PortGroupList;
+    appLogs = new LogsModel(this);
 
     portsWindow = new PortsWindow(pgl, this);
     statsWindow = new PortStatsWindow(pgl, this);
+
     portsDock = new QDockWidget(tr("Ports and Streams"), this);
     portsDock->setObjectName("portsDock");
     portsDock->setFeatures(
                 portsDock->features() & ~QDockWidget::DockWidgetClosable);
+
     statsDock = new QDockWidget(tr("Port Statistics"), this);
     statsDock->setObjectName("statsDock");
     statsDock->setFeatures(
                 statsDock->features() & ~QDockWidget::DockWidgetClosable);
 
+    logsDock_ = new QDockWidget(tr("Logs"), this);
+    logsDock_->setObjectName("logsDock");
+    logsDock_->setFeatures(
+                logsDock_->features() & ~QDockWidget::DockWidgetClosable);
+    logsWindow_ = new LogsWindow(appLogs, logsDock_);
+
     setupUi(this);
 
     menuFile->insertActions(menuFile->actions().at(3), portsWindow->actions());
 
     statsDock->setWidget(statsWindow);
     addDockWidget(Qt::BottomDockWidgetArea, statsDock);
+    logsDock_->setWidget(logsWindow_);
+    addDockWidget(Qt::BottomDockWidgetArea, logsDock_);
+    tabifyDockWidget(statsDock, logsDock_);
+    statsDock->show();
+    statsDock->raise();
+
     portsDock->setWidget(portsWindow);
     addDockWidget(Qt::TopDockWidgetArea, portsDock);
 
@@ -135,10 +154,16 @@ MainWindow::MainWindow(QWidget *parent)
             this, SLOT(onNewVersion(QString)));
     updater->checkForNewVersion();
 
+    // Add the "Local" Port Group
+    if (appParams.optLocalDrone()) {
+        PortGroup *pg = new PortGroup;
+        pgl->addPortGroup(*pg);
+    }
+
     if (appParams.argumentCount()) {
         QString fileName = appParams.argument(0);
         if (QFile::exists(fileName))
-            on_actionOpenSession_triggered(fileName);
+            openSession(fileName);
         else
             QMessageBox::information(NULL, qApp->applicationName(),
                     QString("File not found: " + fileName));
@@ -183,7 +208,7 @@ MainWindow::~MainWindow()
     }
 }
 
-void MainWindow::on_actionOpenSession_triggered(QString fileName)
+void MainWindow::openSession(QString fileName)
 {
     qDebug("Open Session Action (%s)", qPrintable(fileName));
 
@@ -235,6 +260,11 @@ _exit:
     return;
 }
 
+void MainWindow::on_actionOpenSession_triggered()
+{
+    openSession();
+}
+
 void MainWindow::on_actionSaveSession_triggered()
 {
     qDebug("Save Session Action");
@@ -334,6 +364,19 @@ void MainWindow::on_actionHelpOnline_triggered()
     QDesktopServices::openUrl(QUrl(jumpUrl("help", "app", "menu")));
 }
 
+void MainWindow::on_actionDonate_triggered()
+{
+    QDesktopServices::openUrl(QUrl(jumpUrl("donate", "app", "menu")));
+}
+
+void MainWindow::on_actionCheckForUpdates_triggered()
+{
+    Updater *updater = new Updater();
+    connect(updater, SIGNAL(latestVersion(QString)),
+            this, SLOT(onLatestVersion(QString)));
+    updater->checkForNewVersion();
+}
+
 void MainWindow::on_actionHelpAbout_triggered()
 {
     QDialog *aboutDialog = new QDialog;
@@ -405,19 +448,59 @@ void MainWindow::reportLocalServerError()
         tr("<p>If you have remote drone agents running, you can still add "
            "and connect to them.</p>"
            "<p>If you don't want to start the local drone agent at startup, "
-           "provide the <b>-s</b> option to Ostinato on the command line.</p>"
+           "provide the <b>-c</b> option to Ostinato on the command line.</p>"
            "<p>Learn about Ostinato's <a href='%1'>Controller-Agent "
            "architecture</a></p>").arg(jumpUrl("arch")));
 }
 
 void MainWindow::onNewVersion(QString newVersion)
 {
-    QLabel *msg = new QLabel(tr("New Ostinato version %1 available. Visit "
-                   "<a href='%2'>ostinato.org</a> to download")
+    QDate today = QDate::currentDate();
+    QDate lastChecked = QDate::fromString(
+                            appSettings->value(kLastUpdateCheck).toString(),
+                            Qt::ISODate);
+    if (lastChecked.daysTo(today) >= 5) {
+        QMessageBox::information(this, tr("Update check"),
+            tr("<p><b>Ostinato version %1 is now available</b> (you have %2). "
+                "See <a href='%3'>change log</a>.</p>"
+                "<p>Visit <a href='%4'>ostinato.org</a> to download.</p>")
+                .arg(newVersion)
+                .arg(version)
+                .arg(jumpUrl("changelog", "app", "status", "update"))
+                .arg(jumpUrl("download", "app", "status", "update")));
+    }
+    else {
+        QLabel *msg = new QLabel(tr("New Ostinato version %1 available. Visit "
+                    "<a href='%2'>ostinato.org</a> to download")
                 .arg(newVersion)
                 .arg(jumpUrl("download", "app", "status", "update")));
-    msg->setOpenExternalLinks(true);
-    statusBar()->addPermanentWidget(msg);
+        msg->setOpenExternalLinks(true);
+        statusBar()->addPermanentWidget(msg);
+    }
+
+    appSettings->setValue(kLastUpdateCheck, today.toString(Qt::ISODate));
+    sender()->deleteLater();
+}
+
+void MainWindow::onLatestVersion(QString latestVersion)
+{
+    if (version != latestVersion) {
+        QMessageBox::information(this, tr("Update check"),
+            tr("<p><b>Ostinato version %1 is now available</b> (you have %2). "
+                "See <a href='%3'>change log</a>.</p>"
+                "<p>Visit <a href='%4'>ostinato.org</a> to download.</p>")
+                .arg(latestVersion)
+                .arg(version)
+                .arg(jumpUrl("changelog", "app", "status", "update"))
+                .arg(jumpUrl("download", "app", "status", "update")));
+    }
+    else {
+        QMessageBox::information(this, tr("Update check"),
+            tr("You are already running the latest Ostinato version - %1")
+                .arg(version));
+    }
+
+    sender()->deleteLater();
 }
 
 //! Returns true on success (or user cancel) and false on failure
