@@ -20,6 +20,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 #include "portgroup.h"
 
 #include "jumpurl.h"
+#include "log.h"
 #include "settings.h"
 
 #include "emulproto.pb.h"
@@ -166,6 +167,7 @@ void PortGroup::on_rpcChannel_connected()
             new OstProto::VersionCompatibility;
     
     qDebug("connected\n");
+    logInfo(id(), "PortGroup connected");
     emit portGroupDataChanged(mPortGroupId);
 
     reconnectAfter = kMinReconnectWaitTime;
@@ -193,12 +195,16 @@ void PortGroup::processVersionCompatibility(PbRpcController *controller)
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__, 
                 qPrintable(controller->ErrorString()));
+        logError(id(), QString("checkVersion RPC failed: %1")
+                            .arg(controller->ErrorString()));
         goto _error_exit;
     }
 
     if (verCompat->result() == OstProto::VersionCompatibility::kIncompatible) {
         qWarning("incompatible version %s (%s)", version, 
                 qPrintable(QString::fromStdString(verCompat->notes())));
+        logError(id(), QString("checkVersion failed: %1")
+                            .arg(QString::fromStdString(verCompat->notes())));
         compat = kIncompatible;
         emit portGroupDataChanged(mPortGroupId);
 
@@ -236,6 +242,7 @@ _error_exit:
 void PortGroup::on_rpcChannel_disconnected()
 {
     qDebug("disconnected\n");
+    logError(id(), "PortGroup disconnected");
     emit portListAboutToBeChanged(mPortGroupId);
 
     while (!mPorts.isEmpty())
@@ -250,6 +257,8 @@ void PortGroup::on_rpcChannel_disconnected()
     if (reconnect)
     {
         qDebug("starting reconnect timer for %d ms ...", reconnectAfter);
+        logInfo(id(), QString("Reconnect attempt after %1s")
+                            .arg(double(reconnectAfter)/1000.0));
         reconnectTimer->start(reconnectAfter);
     }
 }
@@ -266,6 +275,8 @@ void PortGroup::on_rpcChannel_error(QAbstractSocket::SocketError socketError)
     if ((rpcChannel->state() == QAbstractSocket::UnconnectedState) && reconnect)
     {
         qDebug("starting reconnect timer for %d ms...", reconnectAfter);
+        logInfo(id(), QString("Reconnect attempt after %1s")
+                            .arg(double(reconnectAfter)/1000.0));
         reconnectTimer->start(reconnectAfter);
     }
 }
@@ -296,6 +307,10 @@ void PortGroup::on_rpcChannel_notification(int notifType,
                 qWarning("notif(portConfigChanged) has an empty port_id_list");
                 return;
             }
+            for(int i=0; i < notif->port_id_list().port_id_size(); i++) {
+                logInfo(id(), notif->port_id_list().port_id(i).id(),
+                        QString("Port configuration changed notification"));
+            }
 
             OstProto::PortIdList *portIdList = new OstProto::PortIdList;
             OstProto::PortConfigList *portConfigList = 
@@ -318,6 +333,7 @@ void PortGroup::when_portListChanged(quint32 /*portGroupId*/)
 {
     if (state() == QAbstractSocket::ConnectedState && numPorts() <= 0)
     {
+        logError(id(), QString("No ports in portlist"));
         QMessageBox msgBox;
         msgBox.setIcon(QMessageBox::Warning);
         msgBox.setTextFormat(Qt::RichText);
@@ -348,6 +364,7 @@ void PortGroup::processPortIdList(PbRpcController *controller)
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__, 
                 qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
         goto _error_exit;
     }
 
@@ -403,6 +420,7 @@ void PortGroup::processPortConfigList(PbRpcController *controller)
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__, 
                 qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
         goto _error_exit;
     }
 
@@ -453,6 +471,9 @@ void PortGroup::processPortConfigList(PbRpcController *controller)
                     if (!port->userName().isEmpty() // rsvd?
                             && port->userName() != myself) // by someone else?
                     {
+                        logWarn(id(), j, QString("Port is reserved by %1. "
+                                                    "Skipping reconfiguration")
+                                                 .arg(port->userName()));
                         QString warning =
                                 QString("%1 - %2: %3 is reserved by %4.\n\n"
                                         "Port will not be reconfigured.")
@@ -460,8 +481,8 @@ void PortGroup::processPortConfigList(PbRpcController *controller)
                                     .arg(j)
                                     .arg(port->userAlias())
                                     .arg(port->userName());
-                        QMessageBox::warning(NULL, tr("Open Session"), warning);
                         qWarning("%s", qPrintable(warning));
+                        QMessageBox::warning(NULL, tr("Open Session"), warning);
                         continue;
                     }
                     atConnectPortConfig_[j] = pc;
@@ -481,6 +502,7 @@ void PortGroup::when_configApply(int portIndex)
 {
     OstProto::StreamIdList *streamIdList;
     OstProto::StreamConfigList *streamConfigList;
+    OstProto::BuildConfig *buildConfig;
     OstProto::Ack *ack;
     PbRpcController *controller;
 
@@ -492,14 +514,7 @@ void PortGroup::when_configApply(int portIndex)
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
     mainWindow->setDisabled(true);
 
-    // FIXME: as currently written this code will make unnecessary RPCs
-    // even if the request contains no data; the fix will need to take
-    // care to identify when sync is complete
-    // NOTE: DeviceGroup RPCs are no longer called unnecessarily;
-    // Stream RPCs need to be fixed similarly
-    // Also, drone currently updates its packet list at the end of
-    // modifyStream() implicitly assuming that will be the last API
-    // called - this will also need to be fixed
+    applyTimer_.start();
 
     //
     // Update/Sync DeviceGroups
@@ -513,6 +528,8 @@ void PortGroup::when_configApply(int portIndex)
     deviceGroupIdList->mutable_port_id()->set_id(mPorts[portIndex]->id());
     mPorts[portIndex]->getDeletedDeviceGroupsSinceLastSync(*deviceGroupIdList);
     if (deviceGroupIdList->device_group_id_size()) {
+        logInfo(id(), mPorts[portIndex]->id(),
+                QString("Deleting old DeviceGroups"));
         ack = new OstProto::Ack;
         controller = new PbRpcController(deviceGroupIdList, ack);
         serviceStub->deleteDeviceGroup(controller, deviceGroupIdList, ack,
@@ -528,6 +545,8 @@ void PortGroup::when_configApply(int portIndex)
     deviceGroupIdList->mutable_port_id()->set_id(mPorts[portIndex]->id());
     mPorts[portIndex]->getNewDeviceGroupsSinceLastSync(*deviceGroupIdList);
     if (deviceGroupIdList->device_group_id_size()) {
+        logInfo(id(), mPorts[portIndex]->id(),
+                QString("Creating new DeviceGroups"));
         ack = new OstProto::Ack;
         controller = new PbRpcController(deviceGroupIdList, ack);
         serviceStub->addDeviceGroup(controller, deviceGroupIdList, ack,
@@ -544,6 +563,8 @@ void PortGroup::when_configApply(int portIndex)
     mPorts[portIndex]->getModifiedDeviceGroupsSinceLastSync(
             *deviceGroupConfigList);
     if (deviceGroupConfigList->device_group_size()) {
+        logInfo(id(), mPorts[portIndex]->id(),
+                QString("Modifying changed DeviceGroups"));
         ack = new OstProto::Ack;
         controller = new PbRpcController(deviceGroupConfigList, ack);
         serviceStub->modifyDeviceGroup(controller, deviceGroupConfigList, ack,
@@ -562,49 +583,116 @@ void PortGroup::when_configApply(int portIndex)
     //
     qDebug("applying 'deleted streams' ...");
     streamIdList = new OstProto::StreamIdList;
-    ack = new OstProto::Ack;
-    controller = new PbRpcController(streamIdList, ack);
-
     streamIdList->mutable_port_id()->set_id(mPorts[portIndex]->id());
     mPorts[portIndex]->getDeletedStreamsSinceLastSync(*streamIdList);
-
-    serviceStub->deleteStream(controller, streamIdList, ack,
-        NewCallback(this, &PortGroup::processDeleteStreamAck, controller));
+    if (streamIdList->stream_id_size()) {
+        logInfo(id(), mPorts[portIndex]->id(), QString("Deleting old Streams"));
+        ack = new OstProto::Ack;
+        controller = new PbRpcController(streamIdList, ack);
+        serviceStub->deleteStream(controller, streamIdList, ack,
+            NewCallback(this, &PortGroup::processDeleteStreamAck, controller));
+    }
+    else
+        delete streamIdList;
 
     qDebug("applying 'new streams' ...");
     streamIdList = new OstProto::StreamIdList;
-    ack = new OstProto::Ack;
-    controller = new PbRpcController(streamIdList, ack);
-
     streamIdList->mutable_port_id()->set_id(mPorts[portIndex]->id());
     mPorts[portIndex]->getNewStreamsSinceLastSync(*streamIdList);
-
-    serviceStub->addStream(controller, streamIdList, ack,
-        NewCallback(this, &PortGroup::processAddStreamAck, controller));
+    if (streamIdList->stream_id_size()) {
+        logInfo(id(), mPorts[portIndex]->id(), QString("Creating new Streams"));
+        ack = new OstProto::Ack;
+        controller = new PbRpcController(streamIdList, ack);
+        serviceStub->addStream(controller, streamIdList, ack,
+                NewCallback(this, &PortGroup::processAddStreamAck, controller));
+    }
+    else
+        delete streamIdList;
 
     qDebug("applying 'modified streams' ...");
     streamConfigList = new OstProto::StreamConfigList;
-    ack = new OstProto::Ack;
-    controller = new PbRpcController(streamConfigList, ack);
-
     streamConfigList->mutable_port_id()->set_id(mPorts[portIndex]->id());
     mPorts[portIndex]->getModifiedStreamsSinceLastSync(*streamConfigList);
+    if (streamConfigList->stream_size()) {
+        logInfo(id(), mPorts[portIndex]->id(),
+                QString("Modifying changed Streams"));
+        ack = new OstProto::Ack;
+        controller = new PbRpcController(streamConfigList, ack);
+        serviceStub->modifyStream(controller, streamConfigList, ack,
+                NewCallback(this, &PortGroup::processModifyStreamAck,
+                    portIndex, controller));
+    }
+    else
+        delete streamConfigList;
 
-    serviceStub->modifyStream(controller, streamConfigList, ack,
-            NewCallback(this, &PortGroup::processModifyStreamAck,
+    qDebug("resolve neighbors before building ...");
+    logInfo(id(), mPorts[portIndex]->id(),
+            QString("Resolving device neighbors"));
+    OstProto::PortIdList *portIdList = new OstProto::PortIdList;
+    OstProto::PortId *portId = portIdList->add_port_id();
+    portId->set_id(mPorts[portIndex]->id());
+    ack = new OstProto::Ack;
+    controller = new PbRpcController(portIdList, ack);
+    serviceStub->resolveDeviceNeighbors(controller, portIdList, ack,
+        NewCallback(this, &PortGroup::processResolveDeviceNeighborsAck,
+                    controller));
+
+    qDebug("finish apply by building ...");
+    logInfo(id(), mPorts[portIndex]->id(),
+            QString("Re-building packets"));
+    buildConfig = new OstProto::BuildConfig;
+    ack = new OstProto::Ack;
+    controller = new PbRpcController(buildConfig, ack);
+
+    buildConfig->mutable_port_id()->set_id(mPorts[portIndex]->id());
+    serviceStub->build(controller, buildConfig, ack,
+            NewCallback(this, &PortGroup::processApplyBuildAck,
                 portIndex, controller));
-
 }
 
 void PortGroup::processAddDeviceGroupAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
+    OstProto::DeviceGroupIdList *dgidList
+            = static_cast<OstProto::DeviceGroupIdList*>(controller->request());
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), dgidList->port_id().id(), controller->ErrorString());
+        goto _error_exit;
+    }
+
+    if (ack->status())
+        logError(id(), dgidList->port_id().id(),
+                 QString::fromStdString(ack->notes()));
+
+_error_exit:
     delete controller;
 }
 
 void PortGroup::processDeleteDeviceGroupAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
+    OstProto::DeviceGroupIdList *dgidList
+            = static_cast<OstProto::DeviceGroupIdList*>(controller->request());
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), dgidList->port_id().id(), controller->ErrorString());
+        goto _error_exit;
+    }
+
+    if (ack->status())
+        logError(id(), dgidList->port_id().id(),
+                 QString::fromStdString(ack->notes()));
+
+_error_exit:
     delete controller;
 }
 
@@ -612,18 +700,71 @@ void PortGroup::processModifyDeviceGroupAck(int /*portIndex*/,
         PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
+    OstProto::DeviceGroupIdList *dgidList
+            = static_cast<OstProto::DeviceGroupIdList*>(controller->request());
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), dgidList->port_id().id(), controller->ErrorString());
+        goto _error_exit;
+    }
+
+    if (ack->status())
+        logError(id(), dgidList->port_id().id(),
+                 QString::fromStdString(ack->notes()));
+
+_error_exit:
     delete controller;
 }
 
 void PortGroup::processAddStreamAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
+
+    OstProto::StreamIdList *streamIdList = static_cast<OstProto::StreamIdList*>(
+                                                controller->request());
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), streamIdList->port_id().id(), controller->ErrorString());
+        goto _error_exit;
+    }
+
+    if (ack->status())
+        logError(id(), streamIdList->port_id().id(),
+                 QString::fromStdString(ack->notes()));
+
+_error_exit:
     delete controller;
 }
 
 void PortGroup::processDeleteStreamAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
+
+    OstProto::StreamIdList *streamIdList = static_cast<OstProto::StreamIdList*>(
+                                                controller->request());
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), streamIdList->port_id().id(), controller->ErrorString());
+        goto _error_exit;
+    }
+
+    if (ack->status())
+        logError(id(), streamIdList->port_id().id(),
+                 QString::fromStdString(ack->notes()));
+
+_error_exit:
     delete controller;
 }
 
@@ -632,8 +773,50 @@ void PortGroup::processModifyStreamAck(int portIndex,
 {
     qDebug("In %s", __FUNCTION__);
 
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), mPorts[portIndex]->id(), controller->ErrorString());
+        goto _error_exit;
+    }
+
+    if (ack->status())
+        logError(id(), mPorts[portIndex]->id(),
+                 QString::fromStdString(ack->notes()));
+
+_error_exit:
+    delete controller;
+}
+
+void PortGroup::processApplyBuildAck(int portIndex, PbRpcController *controller)
+{
+    qDebug("In %s", __FUNCTION__);
+
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+
     qDebug("apply completed");
+    logInfo(id(), mPorts[portIndex]->id(),
+            QString("All port changes applied - in %1s")
+                .arg(applyTimer_.elapsed()/1e3));
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), mPorts[portIndex]->id(), controller->ErrorString());
+        goto _error_exit;
+    }
+
+    if (ack->status())
+        logError(id(), mPorts[portIndex]->id(),
+                 QString::fromStdString(ack->notes()));
+
+_error_exit:
     mPorts[portIndex]->when_syncComplete();
+    emit applyFinished();
 
     mainWindow->setEnabled(true);
     QApplication::restoreOverrideCursor();
@@ -682,6 +865,7 @@ void PortGroup::processDeviceList(int portIndex, PbRpcController *controller)
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__,
                 qPrintable(controller->ErrorString()));
+        logError(id(), mPorts[portIndex]->id(), controller->ErrorString());
         goto _exit;
     }
 
@@ -716,6 +900,7 @@ void PortGroup::processDeviceNeighbors(
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__,
                 qPrintable(controller->ErrorString()));
+        logError(id(),  mPorts[portIndex]->id(),controller->ErrorString());
         goto _exit;
     }
 
@@ -752,25 +937,60 @@ void PortGroup::modifyPort(int portIndex, OstProto::Port portConfig)
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
     mainWindow->setDisabled(true);
 
+    logInfo(id(), mPorts[portIndex]->id(),
+            QString("Modifying port configuration"));
     OstProto::Port *port = portConfigList->add_port();
     port->CopyFrom(portConfig);
     port->mutable_port_id()->set_id(mPorts[portIndex]->id());
 
     PbRpcController *controller = new PbRpcController(portConfigList, ack);
     serviceStub->modifyPort(controller, portConfigList, ack, 
-        NewCallback(this, &PortGroup::processModifyPortAck, true, controller));
+        NewCallback(this, &PortGroup::processModifyPortAck, controller));
+
+    logInfo(id(), mPorts[portIndex]->id(),
+            QString("Re-building packets"));
+    OstProto::BuildConfig *buildConfig = new OstProto::BuildConfig;
+    ack = new OstProto::Ack;
+    controller = new PbRpcController(buildConfig, ack);
+    buildConfig->mutable_port_id()->set_id(mPorts[portIndex]->id());
+    serviceStub->build(controller, buildConfig, ack,
+        NewCallback(this, &PortGroup::processModifyPortBuildAck,
+                    true, controller));
 }
 
-void PortGroup::processModifyPortAck(bool restoreUi,PbRpcController *controller)
+void PortGroup::processModifyPortAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
 
     if (controller->Failed())
     {
-        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
                 qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
     }
 
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+
+    delete controller;
+}
+
+void PortGroup::processModifyPortBuildAck(bool restoreUi, PbRpcController *controller)
+{
+    qDebug("In %s", __FUNCTION__);
+
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+    }
+
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+
     if (restoreUi) {
         mainWindow->setEnabled(true);
         QApplication::restoreOverrideCursor();
@@ -787,8 +1007,9 @@ void PortGroup::processUpdatedPortConfig(PbRpcController *controller)
 
     if (controller->Failed())
     {
-        qDebug("%s: rpc failed(%s)", __FUNCTION__, 
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
                 qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
         goto _exit;
     }
 
@@ -837,6 +1058,7 @@ void PortGroup::processStreamIdList(int portIndex, PbRpcController *controller)
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__, 
                 qPrintable(controller->ErrorString()));
+        logError(id(), mPorts[portIndex]->id(), controller->ErrorString());
         goto _exit;
     }
 
@@ -901,7 +1123,7 @@ void PortGroup::processStreamIdList(int portIndex, PbRpcController *controller)
 
             serviceStub->modifyPort(controller, portConfigList, ack,
                 NewCallback(this, &PortGroup::processModifyPortAck,
-                            false, controller));
+                            controller));
         }
 
         // add/modify deviceGroups
@@ -968,6 +1190,15 @@ void PortGroup::processStreamIdList(int portIndex, PbRpcController *controller)
                         portIndex, controller));
         }
 
+        // build packets using the new config
+        OstProto::BuildConfig *buildConfig = new OstProto::BuildConfig;
+        OstProto::Ack *ack = new OstProto::Ack;
+        controller = new PbRpcController(buildConfig, ack);
+        buildConfig->mutable_port_id()->set_id(mPorts[portIndex]->id());
+        serviceStub->build(controller, buildConfig, ack,
+            NewCallback(this, &PortGroup::processModifyPortBuildAck,
+                        false, controller));
+
         // delete newPortConfig
         atConnectPortConfig_[portIndex] = NULL;
 
@@ -1051,6 +1282,7 @@ void PortGroup::processStreamConfigList(int portIndex,
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__, 
                 qPrintable(controller->ErrorString()));
+        logError(id(), mPorts[portIndex]->id(), controller->ErrorString());
         goto _exit;
     }
 
@@ -1121,6 +1353,7 @@ void PortGroup::processDeviceGroupIdList(
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__,
                 qPrintable(controller->ErrorString()));
+        logError(id(), mPorts[portIndex]->id(), controller->ErrorString());
         goto _exit;
     }
 
@@ -1175,8 +1408,11 @@ void PortGroup::getDeviceGroupConfigList(int portIndex)
     using OstProto::DeviceGroupIdList;
     using OstProto::DeviceGroupConfigList;
 
-    if (mPorts[portIndex]->numDeviceGroups() == 0)
+    if (mPorts[portIndex]->numDeviceGroups() == 0) {
+        // No devGrps but we may still have devices (hostDev)
+        getDeviceInfo(portIndex);
         return;
+    }
 
     qDebug("requesting device group config list (port %d) ...", portIndex);
 
@@ -1215,6 +1451,7 @@ void PortGroup::processDeviceGroupConfigList(int portIndex,
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__,
                 qPrintable(controller->ErrorString()));
+        logError(id(), mPorts[portIndex]->id(), controller->ErrorString());
         goto _exit;
     }
 
@@ -1235,8 +1472,7 @@ void PortGroup::processDeviceGroupConfigList(int portIndex,
                                     devGrpCfgList->mutable_device_group(i));
     }
 
-    if (devGrpCfgList->device_group_size())
-        getDeviceInfo(portIndex);
+    getDeviceInfo(portIndex);
 
 #if 0
     // FIXME: incorrect check - will never be true if last port does not have any deviceGroups configured
@@ -1283,6 +1519,19 @@ void PortGroup::processStartTxAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
 
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+        goto _exit;
+    }
+
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+
+_exit:
     delete controller;
 }
 
@@ -1318,6 +1567,19 @@ void PortGroup::processStopTxAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
 
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+        goto _exit;
+    }
+
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+
+_exit:
     delete controller;
 }
 
@@ -1353,6 +1615,17 @@ void PortGroup::processStartCaptureAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
 
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+        goto _exit;
+    }
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+_exit:
     delete controller;
 }
 
@@ -1388,6 +1661,17 @@ void PortGroup::processStopCaptureAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
 
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+        goto _exit;
+    }
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+_exit:
     delete controller;
 }
 
@@ -1412,7 +1696,7 @@ void PortGroup::viewCapture(QList<uint> *portList)
         portId->set_id(portList->at(i));
 
         capFile->open(QIODevice::ReadWrite|QIODevice::Truncate);
-        qDebug("Temp CapFile = %s", capFile->fileName().toAscii().constData());
+        qDebug("Temp CapFile = %s", qPrintable(capFile->fileName()));
         controller->setBinaryBlob(capFile);
 
         serviceStub->getCaptureBuffer(controller, portId, buf,
@@ -1424,6 +1708,8 @@ _exit:
 
 void PortGroup::processViewCaptureAck(PbRpcController *controller)
 {
+    OstProto::PortId *portId = static_cast<OstProto::PortId*>(
+                                                controller->request());
     QFile *capFile = static_cast<QFile*>(controller->binaryBlob());
 
     QString viewer = appSettings->value(kWiresharkPathKey, 
@@ -1431,19 +1717,30 @@ void PortGroup::processViewCaptureAck(PbRpcController *controller)
 
     qDebug("In %s", __FUNCTION__);
 
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), portId->id(), controller->ErrorString());
+        goto _exit;
+    }
+
     capFile->flush();
     capFile->close();
 
     if (!QFile::exists(viewer))
     {
+        logError(QString("Wireshark does not exist at %1").arg(viewer));
         QMessageBox::warning(NULL, "Can't find Wireshark", 
                 viewer + QString(" does not exist!\n\nPlease correct the path"
                 " to Wireshark in the Preferences."));
         goto _exit;
     }
 
-    if (!QProcess::startDetached(viewer, QStringList() << capFile->fileName()))
+    if (!QProcess::startDetached(viewer, QStringList() << capFile->fileName())) {
         qDebug("Failed starting Wireshark");
+        logError(QString("Failed to start %1").arg(viewer));
+    }
 
 _exit:
     delete controller;
@@ -1481,7 +1778,19 @@ _exit:
 void PortGroup::processResolveDeviceNeighborsAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
 
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+        goto _exit;
+    }
+
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+_exit:
     delete controller;
 }
 
@@ -1517,7 +1826,19 @@ _exit:
 void PortGroup::processClearDeviceNeighborsAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
 
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+        goto _exit;
+    }
+
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+_exit:
     delete controller;
 }
 
@@ -1553,6 +1874,7 @@ void PortGroup::processPortStatsList()
     {
         qDebug("%s: rpc failed(%s)", __FUNCTION__, 
                 qPrintable(statsController->ErrorString()));
+        logError(id(), statsController->ErrorString());
         goto _error_exit;
     }
 
@@ -1603,10 +1925,22 @@ _exit:
 void PortGroup::processClearPortStatsAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
 
     // Refresh stats immediately after a stats clear/reset
     getPortStats();
 
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+        goto _exit;
+    }
+
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+_exit:
     delete controller;
 }
 
@@ -1639,6 +1973,17 @@ void PortGroup::processClearStreamStatsAck(PbRpcController *controller)
 {
     qDebug("In %s", __FUNCTION__);
 
+    OstProto::Ack *ack = static_cast<OstProto::Ack*>(controller->response());
+    if (controller->Failed())
+    {
+        qDebug("%s: rpc failed(%s)", __FUNCTION__,
+                qPrintable(controller->ErrorString()));
+        logError(id(), controller->ErrorString());
+        goto _exit;
+    }
+    if (ack->status())
+        logError(id(), QString::fromStdString(ack->notes()));
+_exit:
     delete controller;
 }
 
