@@ -19,6 +19,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "portswindow.h"
 
+#include "applymsg.h"
 #include "deviceswidget.h"
 #include "portconfigdialog.h"
 #include "settings.h"
@@ -28,6 +29,8 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "fileformat.pb.h"
 
+#include "xqlocale.h"
+
 #include <QFileInfo>
 #include <QInputDialog>
 #include <QItemSelectionModel>
@@ -50,6 +53,7 @@ PortsWindow::PortsWindow(PortGroupList *pgl, QWidget *parent)
     plm = pgl;
 
     setupUi(this);
+    applyMsg_ = new ApplyMessage();
     devicesWidget->setPortGroupList(plm);
 
     tvPortList->header()->hide();
@@ -171,6 +175,7 @@ PortsWindow::~PortsWindow()
 {
     delete delegate;
     delete proxyPortModel;
+    delete applyMsg_;
 }
 
 int PortsWindow::portGroupCount()
@@ -375,8 +380,8 @@ void PortsWindow::when_portView_currentChanged(const QModelIndex& currentIndex,
                 updateApplyHint(plm->port(current).portGroupId(),
                                 plm->port(current).id(), true);
             else if (plm->port(current).numStreams())
-                applyHint->setText("Use the Statistics window to transmit "
-                                   "packets");
+                applyHint->setText("Click <img src=':/icons/control_play'/> "
+                                   "to transmit packets");
             else
                 applyHint->setText("");
         }
@@ -425,6 +430,40 @@ void PortsWindow::when_portModel_reset()
     when_portView_currentChanged(QModelIndex(), tvPortList->currentIndex());
 }
 
+void PortsWindow::on_startTx_clicked()
+{
+    QModelIndex current = tvPortList->currentIndex();
+
+    if (proxyPortModel)
+        current = proxyPortModel->mapToSource(current);
+
+    Q_ASSERT(plm->isPort(current));
+
+    QModelIndex curPortGroup = plm->getPortModel()->parent(current);
+    Q_ASSERT(curPortGroup.isValid());
+    Q_ASSERT(plm->isPortGroup(curPortGroup));
+
+    QList<uint> portList({plm->port(current).id()});
+    plm->portGroup(curPortGroup).startTx(&portList);
+}
+
+void PortsWindow::on_stopTx_clicked()
+{
+    QModelIndex current = tvPortList->currentIndex();
+
+    if (proxyPortModel)
+        current = proxyPortModel->mapToSource(current);
+
+    Q_ASSERT(plm->isPort(current));
+
+    QModelIndex curPortGroup = plm->getPortModel()->parent(current);
+    Q_ASSERT(curPortGroup.isValid());
+    Q_ASSERT(plm->isPortGroup(curPortGroup));
+
+    QList<uint> portList({plm->port(current).id()});
+    plm->portGroup(curPortGroup).stopTx(&portList);
+}
+
 void PortsWindow::on_averagePacketsPerSec_editingFinished()
 {
     QModelIndex current = tvPortList->currentIndex();
@@ -435,7 +474,7 @@ void PortsWindow::on_averagePacketsPerSec_editingFinished()
     Q_ASSERT(plm->isPort(current));
 
     bool isOk;
-    double pps = QLocale().toDouble(averagePacketsPerSec->text(), &isOk);
+    double pps = XLocale().toDouble(averagePacketsPerSec->text(), &isOk);
 
     plm->port(current).setAveragePacketRate(pps);
 }
@@ -450,7 +489,7 @@ void PortsWindow::on_averageBitsPerSec_editingFinished()
     Q_ASSERT(plm->isPort(current));
 
     bool isOk;
-    double bps = QLocale().toDouble(averageBitsPerSec->text(), &isOk);
+    double bps = XLocale().toDouble(averageBitsPerSec->text(), &isOk);
 
     plm->port(current).setAverageBitRate(bps);
 }
@@ -519,6 +558,9 @@ void PortsWindow::updateStreamViewActions()
     }
     actionOpen_Streams->setEnabled(plm->isPort(current));
     actionSave_Streams->setEnabled(tvStreamList->model()->rowCount() > 0);
+
+    startTx->setEnabled(tvStreamList->model()->rowCount() > 0);
+    stopTx->setEnabled(tvStreamList->model()->rowCount() > 0);
 }
 
 void PortsWindow::updateApplyHint(int /*portGroupId*/, int /*portId*/,
@@ -528,9 +570,12 @@ void PortsWindow::updateApplyHint(int /*portGroupId*/, int /*portId*/,
         applyHint->setText("Configuration has changed - "
                            "<font color='red'><b>click Apply</b></font> "
                            "to activate the changes");
+    else if (tvStreamList->model()->rowCount() > 0)
+        applyHint->setText("Configuration activated - "
+                           "click <img src=':/icons/control_play'/> "
+                           "to transmit packets");
     else
-        applyHint->setText("Configuration activated. Use the Statistics "
-                           "window to transmit packets");
+        applyHint->setText("Configuration activated");
 }
 
 void PortsWindow::updatePortViewActions(const QModelIndex& currentIndex)
@@ -553,7 +598,7 @@ void PortsWindow::updatePortViewActions(const QModelIndex& currentIndex)
         goto _EXIT;
     }
 
-    qDebug("currentChanged %llx", current.internalId());
+    qDebug("currentChanged %p", (void*)current.internalId());
 
     if (plm->isPortGroup(current))
     {
@@ -645,6 +690,11 @@ void PortsWindow::on_pbApply_clicked()
         goto _exit;
     }
 
+    disconnect(applyMsg_);
+    connect(&(plm->portGroup(curPortGroup)), SIGNAL(applyFinished()),
+            applyMsg_, SLOT(hide()));
+    applyMsg_->show();
+
     // FIXME(HI): shd this be a signal?
     //portGroup.when_configApply(port);
     // FIXME(MED): mixing port id and index!!!
@@ -846,7 +896,7 @@ void PortsWindow::on_actionDuplicate_Stream_triggered()
     if (model->hasSelection())
     {
         bool isOk;
-        int count = QInputDialog::getInteger(this, "Duplicate Streams",
+        int count = QInputDialog::getInt(this, "Duplicate Streams",
                 "Count", 1, 1, 9999, 1, &isOk);
 
         if (!isOk)
