@@ -596,7 +596,8 @@ int AbstractProtocol::protocolFramePayloadSize(int streamIndex) const
   the FrameValue of all the 'frame' fields of the protocol also taking care of 
   fields which are not an integral number of bytes\n 
 */
-QByteArray AbstractProtocol::protocolFrameValue(int streamIndex, bool forCksum) const
+QByteArray AbstractProtocol::protocolFrameValue(int streamIndex, bool forCksum,
+        FrameValueAttrib */*attrib*/) const
 {
     QByteArray proto, field;
     uint bits, lastbitpos = 0;
@@ -620,9 +621,9 @@ QByteArray AbstractProtocol::protocolFrameValue(int streamIndex, bool forCksum)
             else
                 field = fieldData(i, FieldFrameValue, streamIndex).toByteArray();
             qDebug("<<< (%d, %db) %s >>>", proto.size(), lastbitpos,
-                    QString(proto.toHex()).toAscii().constData());
+                    qPrintable(QString(proto.toHex())));
             qDebug("  < %d: (%db/%dB) %s >", i, bits, field.size(),
-                    QString(field.toHex()).toAscii().constData());
+                    qPrintable(QString(field.toHex())));
 
             if (bits == (uint) field.size() * 8)
             {
@@ -930,18 +931,28 @@ quint32 AbstractProtocol::protocolFrameHeaderCksum(int streamIndex,
     CksumType cksumType, CksumScope cksumScope) const
 {
     quint32 sum = 0;
-    quint16 cksum;
+    quint32 cksum;
     AbstractProtocol *p = prev;
 
     Q_ASSERT(cksumType == CksumIpPseudo);
 
+    // We may have extension headers between us and the IP header - skip 'em
     while (p)
     {
         cksum = p->protocolFrameCksum(streamIndex, cksumType);
-        sum += (quint16) ~cksum;
-        qDebug("%s: sum = %u, cksum = %u", __FUNCTION__, sum, cksum);
-        if (cksumScope == CksumScopeAdjacentProtocol)
-            goto out;
+        if (cksum <= 0xFFFF) // protocol has a valid pseudo cksum ie its IP
+        {
+            sum += (quint16) ~cksum;
+            // Ip4/6Protocol::protocolFrameCksum(CksumIpPseudo) only
+            // counts the src/dst IP (see Note in there)
+            // Count the payload length and protocolId here
+            sum += protocolFrameSize(streamIndex)
+                    + protocolFramePayloadSize(streamIndex);
+            sum += protocolId(ProtocolIdIp);
+            qDebug("%s: sum = %x, cksum = %x", __FUNCTION__, sum, cksum);
+            if (cksumScope == CksumScopeAdjacentProtocol)
+                goto out;
+        }
         p = p->prev;
     }
     if (parent)
@@ -1016,6 +1027,17 @@ out:
     return cksum;
 }
 
+/*!
+    Returns true, if the protocol fields are incorrect or may cause
+    overall packet to be invalid
+
+    Error details are put in the optional INOUT param 'errors', if true.
+*/
+bool AbstractProtocol::hasErrors(QStringList* /*errors*/) const
+{
+    return false;
+}
+
 // Stein's binary GCD algo - from wikipedia
 quint64 AbstractProtocol::gcd(quint64 u, quint64 v)
 {
