@@ -19,6 +19,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "streambase.h"
 #include "abstractprotocol.h"
+#include "framevalueattrib.h"
 #include "protocollist.h"
 #include "protocollistiterator.h"
 #include "protocolmanager.h"
@@ -536,7 +537,8 @@ int StreamBase::frameCount() const
 
 // Returns packet length - if bufMaxSize < frameLen(), returns truncated
 // length i.e. bufMaxSize
-int StreamBase::frameValue(uchar *buf, int bufMaxSize, int frameIndex) const
+int StreamBase::frameValue(uchar *buf, int bufMaxSize, int frameIndex,
+        FrameValueAttrib *attrib) const
 {
     int maxSize, size, pktLen, len = 0;
 
@@ -556,10 +558,13 @@ int StreamBase::frameValue(uchar *buf, int bufMaxSize, int frameIndex) const
     while (iter->hasNext())
     {
         AbstractProtocol    *proto;
-        QByteArray            ba;
+        QByteArray          ba;
+        FrameValueAttrib    protoAttrib;
 
         proto = iter->next();
-        ba = proto->protocolFrameValue(frameIndex);
+        ba = proto->protocolFrameValue(frameIndex, false, &protoAttrib);
+        if (attrib)
+            *attrib += protoAttrib;
 
         size = qMin(ba.size(), maxSize-len);
         memcpy(buf+len, ba.constData(), size);
@@ -631,7 +636,57 @@ bool StreamBase::preflightCheck(QStringList &result) const
             break;
     }
 
-    if (frameCount() <= averagePacketRate() && nextWhat() != e_nw_goto_id)
+    ProtocolListIterator *iter = createProtocolListIterator();
+    while (iter->hasNext())
+    {
+        QStringList errors;
+        AbstractProtocol *proto = iter->next();
+        if (proto->hasErrors(&errors)) {
+            result += errors;
+            pass = false;
+        }
+    }
+    delete iter;
+
+    if (isFrameVariable()) {
+        if (frameVariableCount() > frameCount())
+        {
+            if (frameCount() == 1)
+            {
+                result << QObject::tr("Variable fields won't change since "
+                        "only 1 frame%1 is configured to be transmitted - "
+                        "increase number of packets to %L2 to have variable "
+                        "fields change across the configured range")
+                    .arg(sendUnit() == e_su_bursts ?
+                            " (number of bursts * packets per burst)" :
+                            "")
+                    .arg(frameVariableCount());
+                pass = false;
+            }
+            else if (frameCount() > 1)
+            {
+                result << QObject::tr("Variable fields will change for "
+                        "%L1 counts since only %L1 frames%2 are configured "
+                        "to be transmitted - increase number of packets "
+                        "from %L1 to %L3 to have variable fields change "
+                        "across the configured range")
+                    .arg(frameCount())
+                    .arg(sendUnit() == e_su_bursts ?
+                            " (number of bursts * packets per burst)" :
+                            "")
+                    .arg(frameVariableCount());
+                pass = false;
+            }
+        }
+    }
+
+#if 0 // see XXX note below
+    // XXX: This causes false positives for -
+    // * interleaved streams (a port property that we don't have access to)
+    // * pcap imported streams where each stream has only one packet
+    // Ideally we need to get the transmit duration for all the streams
+    // to perform this check
+    if (frameCount() < averagePacketRate() && nextWhat() != e_nw_goto_id)
     {
         result << QObject::tr("Only %L1 frames at the rate of "
                 "%L2 frames/sec are configured to be transmitted. "
@@ -644,6 +699,7 @@ bool StreamBase::preflightCheck(QStringList &result) const
             .arg(frameCount()/averagePacketRate(), 0, 'f');
         pass = false;
     }
+#endif
 
     return pass;
 }
