@@ -41,7 +41,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 static QThreadStorage<QString*> connId;
 
-RpcConnection::RpcConnection(int socketDescriptor, 
+RpcConnection::RpcConnection(qintptr socketDescriptor, 
                              ::google::protobuf::Service *service)
     : socketDescriptor(socketDescriptor),
       service(service)
@@ -59,7 +59,7 @@ RpcConnection::RpcConnection(int socketDescriptor,
 RpcConnection::~RpcConnection()
 { 
     qDebug("destroying connection to %s: %d", 
-            clientSock->peerAddress().toString().toAscii().constData(),
+            qPrintable(clientSock->peerAddress().toString()),
             clientSock->peerPort());
 
     // If still connected, disconnect 
@@ -89,7 +89,7 @@ void RpcConnection::start()
                                       .arg(clientSock->peerPort())));
 
     qDebug("accepting new connection from %s: %d", 
-            clientSock->peerAddress().toString().toAscii().constData(),
+            qPrintable(clientSock->peerAddress().toString()),
             clientSock->peerPort());
     inStream = new google::protobuf::io::CopyingInputStreamAdaptor(
                             new PbQtInputStream(clientSock));
@@ -114,7 +114,9 @@ void RpcConnection::writeHeader(char* header, quint16 type, quint16 method,
     *((quint32*)(header+4)) = qToBigEndian(length);
 }
 
-void RpcConnection::sendRpcReply(PbRpcController *controller)
+void RpcConnection::sendRpcReply(
+        const ::google::protobuf::MethodDescriptor *method,
+        PbRpcController *controller)
 {
     google::protobuf::Message *response = controller->response();
     QIODevice *blob;
@@ -178,8 +180,10 @@ void RpcConnection::sendRpcReply(PbRpcController *controller)
         qDebug("Server(%s): sending %d bytes to client <----",
             __FUNCTION__, len + PB_HDR_SIZE);
         BUFDUMP(msg, 8);
-        qDebug("method = %d\nreq = \n%s---->", 
-            pendingMethodId, response->DebugString().c_str());
+        qDebug("method = %d:%s\nresp = %s\n%s---->",
+            pendingMethodId, method ? method->name().c_str() : "",
+            method ? method->output_type()->name().c_str() : "",
+            response->DebugString().c_str());
     }
 
     clientSock->write(msg, PB_HDR_SIZE);
@@ -240,7 +244,7 @@ void RpcConnection::sendNotification(int notifType,
 void RpcConnection::on_clientSock_disconnected()
 {
     qDebug("connection closed from %s: %d",
-            clientSock->peerAddress().toString().toAscii().constData(),
+            qPrintable(clientSock->peerAddress().toString()),
             clientSock->peerPort());
 
     deleteLater();
@@ -249,8 +253,7 @@ void RpcConnection::on_clientSock_disconnected()
 
 void RpcConnection::on_clientSock_error(QAbstractSocket::SocketError socketError)
 {
-    qDebug("%s (%d)", clientSock->errorString().toAscii().constData(), 
-            socketError);
+    qDebug("%s (%d)", qPrintable(clientSock->errorString()), socketError);
 }
 
 void RpcConnection::on_clientSock_dataAvail()
@@ -344,16 +347,19 @@ void RpcConnection::on_clientSock_dataAvail()
         bool ok = req->ParseFromBoundedZeroCopyStream(inStream, len);
         if (!ok)
             qWarning("ParseFromBoundedZeroCopyStream fail "
-                     "for method %d and len %d", method, len);
+                     "for method %d:%s and len %d",
+                     method, methodDesc->name().c_str(),len);
     }
 
     if (!req->IsInitialized())
     {
         qWarning("Missing required fields in request <----");
-        qDebug("method = %d\n"
-               "req = \n%s"
+        qDebug("method = %d:%s\n"
+               "req = %s\n%s"
                "missing = \n%s----->",
-                method, req->DebugString().c_str(),
+                method, methodDesc->name().c_str(),
+                methodDesc->input_type()->name().c_str(),
+                req->DebugString().c_str(),
                 req->InitializationErrorString().c_str());
         error = QString("RPC %1() missing required fields in request - %2")
                     .arg(QString::fromStdString(
@@ -368,9 +374,10 @@ void RpcConnection::on_clientSock_dataAvail()
     
     if (method != 13) {
         qDebug("Server(%s): successfully received/parsed msg <----", __FUNCTION__);
-        qDebug("method = %d\n"
-               "req = \n%s---->",
-                method,
+        qDebug("method = %d:%s\n"
+               "req = %s\n%s---->",
+                method, methodDesc->name().c_str(),
+                methodDesc->input_type()->name().c_str(),
                 req->DebugString().c_str());
     }
 
@@ -380,7 +387,7 @@ void RpcConnection::on_clientSock_dataAvail()
 
     service->CallMethod(methodDesc, controller, req, resp,
         google::protobuf::NewCallback(this, &RpcConnection::sendRpcReply, 
-                                      controller));
+                                      methodDesc, controller));
 
     return;
 
@@ -395,11 +402,12 @@ _error_exit2:
     controller->SetFailed(error);
     if (disconnect)
         controller->TriggerDisconnect();
-    sendRpcReply(controller);
+    sendRpcReply(methodDesc, controller);
     return;
 }
 
-void RpcConnection::connIdMsgHandler(QtMsgType /*type*/, const char* msg)
+void RpcConnection::connIdMsgHandler(QtMsgType /*type*/,
+        const QMessageLogContext &/*context*/, const QString &msg)
 {
     if (connId.hasLocalData()) {
         QString newMsg(*connId.localData());
@@ -410,6 +418,6 @@ void RpcConnection::connIdMsgHandler(QtMsgType /*type*/, const char* msg)
         return;
     }
 
-    fprintf(stderr, "%s\n", msg);
+    fprintf(stderr, "%s\n", qPrintable(msg));
     fflush(stderr);
 }
