@@ -23,19 +23,12 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include <QThread>
 
-// FIXME: QThreadX till we change minimum version of Qt from Qt4.3+ to Qt4.4+
-class QThreadX: public QThread
-{
-protected:
-    virtual ~QThreadX() { qDebug("QThreadX going down!"); }
-    void run() { exec(); }
-};
-
-RpcServer::RpcServer()
+RpcServer::RpcServer(bool perConnLogs)
 {
     service = NULL; 
 
-    qInstallMsgHandler(RpcConnection::connIdMsgHandler);
+    if (perConnLogs)
+        qInstallMessageHandler(RpcConnection::connIdMsgHandler);
 }
 
 RpcServer::~RpcServer()
@@ -51,19 +44,19 @@ bool RpcServer::registerService(::google::protobuf::Service *service,
     {
         qDebug("Unable to start the server on <%s>: %s",
                 qPrintable(address.toString()),
-                errorString().toAscii().constData());
+                qPrintable(errorString()));
         return false;
     }
 
     qDebug("The server is running on %s: %d", 
-            serverAddress().toString().toAscii().constData(),
+            qPrintable(serverAddress().toString()),
             serverPort());
     return true;
 }
 
-void RpcServer::incomingConnection(int socketDescriptor)
+void RpcServer::incomingConnection(qintptr socketDescriptor)
 {
-    QThread *thread = new QThreadX; // FIXME:QThreadX pending Qt4.4+
+    QThread *thread = new QThread;
     RpcConnection *conn = new RpcConnection(socketDescriptor, service);
 
     conn->moveToThread(thread);
