@@ -63,6 +63,8 @@ PcapPort::PcapPort(int id, const char *device)
 
 void PcapPort::init()
 {
+    AbstractPort::init();
+
     if (!monitorTx_->isDirectional())
         transmitter_->useExternalStats(&stats_);
 
@@ -350,14 +352,14 @@ void PcapPort::PortMonitor::stop()
  */
 PcapPort::PortCapturer::PortCapturer(const char *device)
 {
-    device_ = QString::fromAscii(device);
+    device_ = QString::fromLatin1(device);
     stop_ = false;
     state_ = kNotStarted;
 
     if (!capFile_.open())
         qWarning("Unable to open temp cap file");
 
-    qDebug("cap file = %s", capFile_.fileName().toAscii().constData());
+    qDebug("cap file = %s", qPrintable(capFile_.fileName()));
 
     dumpHandle_ = NULL;
     handle_ = NULL;
@@ -381,7 +383,7 @@ void PcapPort::PortCapturer::run()
         goto _exit;
     }
 _retry:
-    handle_ = pcap_open_live(device_.toAscii().constData(), 65535, 
+    handle_ = pcap_open_live(qPrintable(device_), 65535,
                     flag, 1000 /* ms */, errbuf);
 
     if (handle_ == NULL)
@@ -389,20 +391,20 @@ _retry:
         if (flag && QString(errbuf).contains("promiscuous"))
         {
             qDebug("%s:can't set promiscuous mode, trying non-promisc", 
-                    device_.toAscii().constData());
+                    qPrintable(device_));
             flag = 0;
             goto _retry;
         }
         else
         {
             qDebug("%s: Error opening port %s: %s\n", __FUNCTION__,
-                    device_.toAscii().constData(), errbuf);
+                    qPrintable(device_), errbuf);
             goto _exit;
         }
     }
 
-    dumpHandle_ = pcap_dump_open(handle_, 
-            capFile_.fileName().toAscii().constData());
+    dumpHandle_ = pcap_dump_open(handle_, qPrintable(capFile_.fileName()));
+    PcapSession::preRun();
     state_ = kRunning;
     while (1)
     {
@@ -424,16 +426,21 @@ _retry:
                         __PRETTY_FUNCTION__, ret, pcap_geterr(handle_));
                 break;
             case -2:
+                qDebug("Loop/signal break or some other error");
+                break;
             default:
-                qFatal("%s: Unexpected return value %d", __PRETTY_FUNCTION__, ret);
+                qWarning("%s: Unexpected return value %d", __PRETTY_FUNCTION__, ret);
+                stop_ = true;
         }
 
         if (stop_) 
         {
-            qDebug("user requested capture stop\n");
+            qDebug("user requested capture stop");
             break;
         }
     }
+    PcapSession::postRun();
+
     pcap_dump_close(dumpHandle_);
     pcap_close(handle_);
     dumpHandle_ = NULL;
@@ -463,6 +470,7 @@ void PcapPort::PortCapturer::stop()
 {
     if (state_ == kRunning) {
         stop_ = true;
+        PcapSession::stop(handle_);
         while (state_ == kRunning)
             QThread::msleep(10);
     }
@@ -492,7 +500,7 @@ QFile* PcapPort::PortCapturer::captureFile()
 PcapPort::EmulationTransceiver::EmulationTransceiver(const char *device,
         DeviceManager *deviceManager)
 {
-    device_ = QString::fromAscii(device);
+    device_ = QString::fromLatin1(device);
     deviceManager_ = deviceManager;
     stop_ = false;
     state_ = kNotStarted;
@@ -565,7 +573,7 @@ _retry:
     {
         if (flags && QString(errbuf).contains("promiscuous"))
         {
-            notify("Unable to set promiscuous mode on <%s> - "
+            Xnotify("Unable to set promiscuous mode on <%s> - "
                     "device emulation will not work", qPrintable(device_));
             goto _exit;
         }
@@ -580,7 +588,7 @@ _retry:
 #endif
         else
         {
-            notify("Unable to open <%s> [%s] - device emulation will not work",
+            Xnotify("Unable to open <%s> [%s] - device emulation will not work",
                     qPrintable(device_), errbuf);
             goto _exit;
         }
@@ -605,6 +613,7 @@ _retry:
     }
 
 _skip_filter:
+    PcapSession::preRun();
     state_ = kRunning;
     while (1)
     {
@@ -642,17 +651,22 @@ _skip_filter:
                         __PRETTY_FUNCTION__, ret, pcap_geterr(handle_));
                 break;
             case -2:
+                qDebug("Loop/signal break or some other error");
+                break;
             default:
-                qFatal("%s: Unexpected return value %d", __PRETTY_FUNCTION__,
+                qWarning("%s: Unexpected return value %d", __PRETTY_FUNCTION__,
                         ret);
+                stop_ = true;
         }
 
         if (stop_)
         {
-            qDebug("user requested receiver stop\n");
+            qDebug("user requested receiver stop");
             break;
         }
     }
+    PcapSession::postRun();
+
     pcap_close(handle_);
     handle_ = NULL;
     stop_ = false;
@@ -679,6 +693,7 @@ void PcapPort::EmulationTransceiver::stop()
 {
     if (state_ == kRunning) {
         stop_ = true;
+        PcapSession::stop(handle_);
         while (state_ == kRunning)
             QThread::msleep(10);
     }
