@@ -19,16 +19,23 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>
 
 #include "winpcapport.h"
 
+#include "interfaceinfo.h"
+
 #include <QCoreApplication> 
 #include <QProcess> 
 
 #ifdef Q_OS_WIN32
 
+#include <ws2ipdef.h>
+
+PIP_ADAPTER_ADDRESSES WinPcapPort::adapterList_ = NULL;
 const uint OID_GEN_MEDIA_CONNECT_STATUS = 0x00010114;
 
-WinPcapPort::WinPcapPort(int id, const char *device)
-    : PcapPort(id, device) 
+WinPcapPort::WinPcapPort(int id, const char *device, const char *description)
+    : PcapPort(id, device)
 {
+    populateInterfaceInfo();
+
     monitorRx_->stop();
     monitorTx_->stop();
     monitorRx_->wait();
@@ -40,6 +47,8 @@ WinPcapPort::WinPcapPort(int id, const char *device)
     monitorRx_ = new PortMonitor(device, kDirectionRx, &stats_);
     monitorTx_ = new PortMonitor(device, kDirectionTx, &stats_);
 
+    data_.set_description(description);
+
     adapter_ = PacketOpenAdapter((CHAR*)device);
     if (!adapter_)
         qFatal("Unable to open adapter %s", device);
@@ -88,7 +97,7 @@ bool WinPcapPort::hasExclusiveControl()
                 + "/bindconfig.exe");
     int exitCode;
 
-    qDebug("%s: %s", __FUNCTION__, portName.toAscii().constData());
+    qDebug("%s: %s", __FUNCTION__, qPrintable(portName));
 
     if (!QFile::exists(bindConfigFilePath))
         return false;
@@ -111,7 +120,7 @@ bool WinPcapPort::setExclusiveControl(bool exclusive)
                 + "/bindconfig.exe");
     QString status;
 
-    qDebug("%s: %s", __FUNCTION__, portName.toAscii().constData());
+    qDebug("%s: %s", __FUNCTION__, qPrintable(portName));
 
     if (!QFile::exists(bindConfigFilePath))
         return false;
@@ -170,8 +179,8 @@ void WinPcapPort::PortMonitor::run()
                 case kDirectionRx:
                     stats_->rxPkts += pkts;
                     stats_->rxBytes += bytes;
-                    stats_->rxPps = (pkts  * 1000000) / usec;
-                    stats_->rxBps = (bytes * 1000000) / usec;
+                    stats_->rxPps = qRound64(pkts  * 1e6 / usec);
+                    stats_->rxBps = qRound64(bytes * 1e6 / usec);
                     break;
 
                 case kDirectionTx:
@@ -192,8 +201,8 @@ void WinPcapPort::PortMonitor::run()
                         lastTxPkts = txPkts;
                         lastTxBytes = txBytes;
                     }
-                    stats_->txPps = (pkts  * 1000000) / usec;
-                    stats_->txBps = (bytes * 1000000) / usec;
+                    stats_->txPps = qRound64(pkts  * 1e6 / usec);
+                    stats_->txBps = qRound64(bytes * 1e6 / usec);
                     break;
 
                 default:
@@ -223,4 +232,111 @@ void WinPcapPort::PortMonitor::run()
     }
 }
 
+void WinPcapPort::populateInterfaceInfo()
+{
+    if (!adapterList_) {
+        qWarning("Adapter List not available");
+        return;
+    }
+
+    PIP_ADAPTER_ADDRESSES adapter = adapterList_;
+    while (adapter && !QString(name()).endsWith(QString(adapter->AdapterName)))
+        adapter = adapter->Next;
+
+    if (!adapter) {
+        qWarning("Adapter info not found for %s", name());
+        return;
+    }
+
+    interfaceInfo_ = new InterfaceInfo;
+    if (adapter->PhysicalAddressLength == 6) {
+        interfaceInfo_->mac = qFromBigEndian<quint64>(
+                                    adapter->PhysicalAddress) >> 16;
+    }
+    else
+        interfaceInfo_->mac = 0;
+
+#define SOCKET_ADDRESS_FAMILY(x) \
+    (x.lpSockaddr->sa_family)
+
+#define SOCKET_ADDRESS_IP4(x) \
+    (qFromBigEndian<quint32>(((sockaddr_in*)(x.lpSockaddr))->sin_addr.S_un.S_addr));
+
+#define SOCKET_ADDRESS_IP6(x) \
+    (UInt128(((PSOCKADDR_IN6)(x.lpSockaddr))->sin6_addr.u.Byte));
+
+    // We may have multiple gateways - use the first for each family
+    quint32 ip4Gateway = 0;
+    PIP_ADAPTER_GATEWAY_ADDRESS gateway = adapter->FirstGatewayAddress;
+    while (gateway) {
+        if (SOCKET_ADDRESS_FAMILY(gateway->Address) == AF_INET) {
+            ip4Gateway = SOCKET_ADDRESS_IP4(gateway->Address);
+            break;
+        }
+        gateway = gateway->Next;
+    }
+    UInt128 ip6Gateway(0, 0);
+    gateway = adapter->FirstGatewayAddress;
+    while (gateway) {
+        if (SOCKET_ADDRESS_FAMILY(gateway->Address) == AF_INET6) {
+            ip6Gateway = SOCKET_ADDRESS_IP6(gateway->Address);
+            break;
+        }
+        gateway = gateway->Next;
+    }
+
+    PIP_ADAPTER_UNICAST_ADDRESS ucast = adapter->FirstUnicastAddress;
+    while (ucast) {
+        if (SOCKET_ADDRESS_FAMILY(ucast->Address) == AF_INET) {
+            Ip4Config ip;
+            ip.address = SOCKET_ADDRESS_IP4(ucast->Address);
+            ip.prefixLength = ucast->OnLinkPrefixLength;
+            ip.gateway = ip4Gateway;
+            interfaceInfo_->ip4.append(ip);
+        }
+        else if (SOCKET_ADDRESS_FAMILY(ucast->Address) == AF_INET6) {
+            Ip6Config ip;
+            ip.address = SOCKET_ADDRESS_IP6(ucast->Address);
+            ip.prefixLength = ucast->OnLinkPrefixLength;
+            ip.gateway = ip6Gateway;
+            interfaceInfo_->ip6.append(ip);
+        }
+        ucast = ucast->Next;
+    }
+#undef SOCKET_ADDRESS_FAMILY
+#undef SOCKET_ADDRESS_IP4
+#undef SOCKET_ADDRESS_IP6
+}
+
+void WinPcapPort::fetchHostNetworkInfo()
+{
+    DWORD ret;
+    ULONG bufLen = 15*1024; // MS recommended starting size
+
+    while (1) {
+        adapterList_ = (IP_ADAPTER_ADDRESSES *) malloc(bufLen);
+        ret = GetAdaptersAddresses(AF_UNSPEC,
+                                   GAA_FLAG_INCLUDE_ALL_INTERFACES
+                                   | GAA_FLAG_INCLUDE_GATEWAYS,
+                                   0, adapterList_, &bufLen);
+        if (ret == ERROR_BUFFER_OVERFLOW) {
+            free(adapterList_);
+            continue;
+        }
+        break;
+    }
+
+    if (ret != NO_ERROR) {
+        free(adapterList_);
+        adapterList_ = NULL;
+        return;
+    }
+}
+
+void WinPcapPort::freeHostNetworkInfo()
+{
+    free(adapterList_);
+    adapterList_ = NULL;
+}
+
 #endif
